#include <string>
#include <fmt/printf.h>
#include "cxxopts.hpp"
#include "kyut/EmbeddingMethod.hpp"

namespace {
    const std::string command = "snpi";
    const std::string version = "0.1.0";
    const std::string description = "WebAssembly watermark embedder";

    struct Options {
        std::string input;
        std::string output;
        kyut::EmbeddingMethod method;
    };

    Options parse_options(int argc, char** argv) {
        cxxopts::Options options{command, fmt::format("{} - {}", command, description)};

        // clang-format off
        options
            .add_options()
            ("v,version", "Show version")
            ("h,help", "Print help")
            ("o,output", "Output file (required)", cxxopts::value<std::string>(), "OUTPUT")
            ("m,method", "Embedding method (required)", cxxopts::value<std::string>(), "function-ordering, operand-swapping")
            ("input", "Input file", cxxopts::value<std::string>())
            ;
        // clang-format on

        options
            .positional_help("INPUT")
            .parse_positional("input");

        try {
            const auto result = options.parse(argc, const_cast<const char**&>(argv));

            if (result.count("version") > 0) {
                // Show version.
                fmt::print("{} v{}\n", command, version);
                std::exit(EXIT_SUCCESS);
            }

            if (result.count("help") > 0) {
                // Print help.
                fmt::print("{}\n", options.help());
                std::exit(EXIT_SUCCESS);
            }

            if (result.count("input") == 0) {
                throw std::runtime_error{"no input file"};
            }

            if (result.count("output") == 0) {
                throw std::runtime_error{"no output file"};
            }

            if (result.count("method") == 0) {
                throw std::runtime_error{"no embedding method"};
            }

            const auto input = result["input"].as<std::string>();
            const auto output = result["output"].as<std::string>();
            const auto method_text = result["method"].as<std::string>();
            const auto method = kyut::embedding_method_from(method_text)
                                    .value_or_eval([method_text]() -> kyut::EmbeddingMethod {
                                        throw std::runtime_error{fmt::format("unknown embedding method: {}", method_text)};
                                    });

            return Options{
                input,
                output,
                method,
            };
        } catch (const std::exception& e) {
            fmt::print(std::cerr, "{}\n", e.what());
            std::exit(EXIT_FAILURE);
        }
    }
} // namespace

int main(int argc, char* argv[]) {
    const auto opts = parse_options(argc, argv);
}
